<?php

/**
 * generateNotes.php: Generates Git commit notes giving SVN revision numbers
 *                    for the old MediaWiki repositories.
 *
 * Written by Kevin Israel. Released to the public domain.
 *
 * Command used to retrieve SVN log:
 *   svn log --xml http://svn.wikimedia.org/svnroot/mediawiki > svnlog.xml
 * Command used to retrieve Git log:
 *   git log --all --format='%H%x09%ae%x09%at' > gitlog.tsv
 * Command used to retrieve USERINFO files:
 *   svn co http://svn.wikimedia.org/svnroot/mediawiki/USERINFO
 *
 * Redirect standard input to the JSON configuration file.
 * Pipe standard output to git fast-import.
 */

require_once 'generateNotesFunctions.php';

// Load configuration
$config = json_decode( stream_get_contents( STDIN ), true );

// Set the timezone
if ( isset( $config['commitTimezone'] ) ) {
	date_default_timezone_set( $config['commitTimezone'] );
}

// Ensure git-fast-import can handle this program's output
echo "feature notes\n";

// Read logs and aliases from both systems
if ( isset( $config['svnSerializedLogPath'] ) ) {
	$svnLog = unserialize( file_get_contents( $config['svnSerializedLogPath'] ) );
} else {
	fwrite( STDERR, "Reading SVN log..." );
	$svnLog = readSvnLog( $config['svnLogPath'] );
	fwrite( STDERR, " done.\n" );
}

fwrite( STDERR, "Reading Git log..." );
$gitLog = readGitLog( $config['gitLogPath'] );

fwrite( STDERR, " done.\nReading USERINFO files..." );
$aliases = readUserAliases( $config['userInfoPath'] );

// Deal with aliases defined in SVN USERINFO files
fwrite( STDERR, " done.\nMapping SVN usernames..." );
foreach ( $svnLog as &$svnLogEntry ) {
	if ( isset( $aliases[$svnLogEntry['user']] ) ) {
		$svnLogEntry['user'] = $aliases[$svnLogEntry['user']];
	}
}

// Some aliases were not used at the time of conversion
fwrite( STDERR, " done.\nMapping Git emails..." );
foreach ( $gitLog as &$gitLogEntry ) {
	$user = preg_replace( "/{$config['emailFormat']}/", '$1', $gitLogEntry['user'] );
	if ( isset( $aliases[$user] ) ) {
		$user = $aliases[$user];
	}
	$gitLogEntry['user'] = $user;
}

// Rearrange the logs for easy pairing of revisions
fwrite( STDERR, " done.\nRearranging SVN log..." );
$svnRearranged = rearrangeLog( $svnLog );

fwrite( STDERR, " done.\nRearranging Git log..." );
$gitRearranged = rearrangeLog( $gitLog );

// Create the notes
fwrite( STDERR, " done.\nCreating note blobs..." );

$noteMods = '';
$markNum = 1;
foreach ( $svnRearranged as $key => $svnRevs ) {
	if ( !isset( $gitRearranged[$key] ) ) {
		continue;
	}

	// Generate note text and write it out as a blob
	$text = '';
	foreach ( $svnRevs as $svnRev ) {
		$text .= str_replace( '$1', $svnRev, $config['noteFormat'] );
	}
	echo "blob\nmark :$markNum\n";
	echo formatGitData( $text );

	// Associate note text with each corresponding Git commit
	foreach ( $gitRearranged[$key] as $gitRev ) {
		$noteMods .= "N :$markNum $gitRev\n";
	}

	$markNum++;
}

// Make the commit
fwrite( STDERR, " done.\nMaking commit..." );

echo "commit {$config['commitRef']}\n";
echo "committer {$config['commitName']} <{$config['commitEmail']}> " . date( 'U O' ) . "\n";
echo formatGitData( $config['commitMessage'] );
echo $noteMods;

fwrite( STDERR, " done.\n" );
